#include "GLNormalRenderer.h"
#include "GLNormalRendererGeometry3DVisitor.h"
#include "GLNormalRendererGroupingVisitor.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace GL {

NormalRenderer::NormalRenderer()
{
  // Getting the GlobalVariables to initialize its values.
  globalVariables = GVManager::getInstanceOf<NormalRendererGlobalVariables>();

  // The algorithm for drawing the normals is based upon a graphTraversal traversal of the 
  // GL scene graph.
  graphTraversal = new DFSGraphTraversal();
  
  // Setting the component visitor to the tree traversal algorithm.
  graphTraversal->setComponentVisitor(new NormalRendererGeometry3DVisitor());
  graphTraversal->setComponentVisitor(new NormalRendererGroupingVisitor());
}

NormalRenderer::~NormalRenderer()
{
  // Removing the GlobalVariables.
  GVManager::removeInstanceOf<NormalRendererGlobalVariables>();

  delete graphTraversal;
}

void NormalRenderer::setNormalLength(float value)
{
  globalVariables->setNormalLength(value);
}

void NormalRenderer::render(SFNode N) const
{
  glDisable(GL_COLOR_MATERIAL);
  graphTraversal->traverse(N);
}

}
}
