////////////////////////////////////////////////////////////////////////////////
///                            MeshTransformComputerGlobalVariables.h        ///
////////////////////////////////////////////////////////////////////////////////

#ifndef MESHTRANSFORMCOMPUTERGLOBALVARIABLES_H
#define MESHTRANSFORMCOMPUTERGLOBALVARIABLES_H

#include <X3DTK/kernel.h>

#include <list>

namespace X3DTK {
namespace MESH {

/// GlobalVariables for the MeshTransformComputer module.

class MeshTransformComputerGlobalVariables : public GlobalVariables
{
public:
  /// Constructor.
  MeshTransformComputerGlobalVariables();
  /// Destructor.
  virtual ~MeshTransformComputerGlobalVariables();

  /// Initializes the traversal.
  void init();
  /// Finishes the traversal.
  void finish();

  /// Pushes the transformation matrix.
  void pushMatrix(const SFMatrix34f &transformation);  
  
  /// Pops the transformation matrix.
  void popMatrix();
  
  /// Gets the current transformation matrix.
  SFMatrix34f getMatrix() const {return _matrixStack.front();};

private: 
  std::list<SFMatrix34f> _matrixStack;
};

}
}

#endif
