#include "MyStructureComputerCoreVisitor.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

MyStructureComputerCoreVisitor::MyStructureComputerCoreVisitor()
{
  // Enter functions.
  defineNewEnterFunction(&MyStructureComputerCoreVisitor::enterMesh);
  defineNewEnterFunction(&MyStructureComputerCoreVisitor::enterTransform);
  
  // Leave functions.
  defineNewLeaveFunction(&MyStructureComputerCoreVisitor::leaveX3DGroupingNode);
  
  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<MyStructureComputerGlobalVariables>();
}

MyStructureComputerCoreVisitor::~MyStructureComputerCoreVisitor()
{
}

void MyStructureComputerCoreVisitor::enterMesh(Mesh *M) const
{
  SFMatrix34f T = globalVariables->getMatrix();

  globalVariables->beginNewMesh();
  
  // filling the vertices.
  for (Mesh::MVertex::const_iterator it = M->getVertices().begin(); it != M->getVertices().end(); ++it)
  {
    SFPoint3f P = T*(*it)->data().getPoint();
    globalVariables->addVertex(P.x, P.y, P.z);
  }
  
  // filling the faces.
  for (Mesh::MFace::const_iterator it = M->getFaces().begin(); it != M->getFaces().end(); ++it)
  {
    Mesh::MEdge edges = (*it)->getEdges();
    list<unsigned int> indexList;
    for (Mesh::MEdge::const_iterator itEdge = edges.begin(); itEdge != edges.end(); ++itEdge)
      indexList.push_back((*itEdge)->getFromVertex()->getIndex());
    
    globalVariables->addFace(indexList);
  }  
}

void MyStructureComputerCoreVisitor::enterTransform(Transform *T) const
{
  globalVariables->pushMatrix(T->getTransform());
}

void MyStructureComputerCoreVisitor::leaveX3DGroupingNode(X3DGroupingNode *N) const
{
  globalVariables->popMatrix();
}

}
}
