////////////////////////////////////////////////////////////////////////////////
///                            MyStructureComputerGlobalVariables.h          ///
////////////////////////////////////////////////////////////////////////////////

#ifndef MYSTRUCTURECOMPUTERGLOBALVARIABLES_H
#define MYSTRUCTURECOMPUTERGLOBALVARIABLES_H

#include "MySimpleMesh.h"

#include <X3DTK/kernel.h>

#include <list>

namespace X3DTK {
namespace MESH {

/// GlobalVariables for the MyStructureComputer module.

class MyStructureComputerGlobalVariables : public GlobalVariables
{
public:
  /// Constructor.
  MyStructureComputerGlobalVariables();
  /// Destructor.
  virtual ~MyStructureComputerGlobalVariables();

  /// Initializes the traversal.
  void init();
  /// Finishes the traversal.
  void finish();

  void beginNewMesh();
  inline void addVertex(float x, float y, float z) {_mesh->addVertex(x, y, z);};
  void addFace(const Face &face);
  /// Pushes the transformation matrix.
  void pushMatrix(const SFMatrix34f &transformation);  
  /// Pops the transformation matrix.
  void popMatrix();

  inline MySimpleMesh *getMesh() const {return _mesh;};  
  /// Gets the current transformation matrix.
  SFMatrix34f getMatrix() const {return _matrixStack.front();};

private: 
  MySimpleMesh *_mesh;
  unsigned int _decal;
  std::list<SFMatrix34f> _matrixStack;
};

}
}

#endif
