/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            BBoxUpdaterGlobalVariables.h                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef BBOXUPDATERGLOBALVARIABLES_H
#define BBOXUPDATERGLOBALVARIABLES_H

#include "X3DSceneGraphTypes.h"
#include "GlobalVariables.h"
#include "BBox.h"

#include <list>

namespace X3DTK {
namespace X3D {

/// GlobalVariables for the BBoxUpdater module.

class BBoxUpdaterGlobalVariables : public GlobalVariables
{
public:
  /// Constructor.
  BBoxUpdaterGlobalVariables();
  /// Destructor.
  virtual ~BBoxUpdaterGlobalVariables();
  
  /// Finishes the traversal.
  virtual void finish();
  
  /// Sets the staticProcessing.
  void setStaticProcessing(bool value);
  /// Adds the Bounding Box BB.
  void addBBox(SFNode N, BBox *BB);
  /// Sets the current BBox (for the Geometry nodes of Shape).
  void setShapeBBox(const BBox &BB);
  /// Adds the bounding Box to the list.
  void addBBoxToMergeList(const BBox &BB);
  
  /// Gets the staticProcessing value.
  inline bool getStaticProcessing() const {return _staticProcessing;};
  /// Gets the current BBox.
  BBox getShapeBBox() const {return _shapeBBox;};
  /// Gets the BBox of N.
  BBox *getBBox(SFNode N) const;
  /// Merges the BBox of the list.
  BBox mergeBBox();
  
private:
  bool _staticProcessing;
  BBox _shapeBBox;
  std::list<std::pair<SFNode, BBox *> > _BBList;
  std::list<BBox> _BBToMergeList;  
};

}
}

#endif
