/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            Cone.h                                          //
////////////////////////////////////////////////////////////////////////////////

#ifndef CONE_H
#define CONE_H

#include "X3DTypes.h"
#include "X3DGeometry3DNode.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/geometry3D.html">
/// Geometry3D component</a> defining a Cone
/// (<a href="../../doc/X3DSpecs/Part01/components/geometry3D.html#Cone">
/// documentation</a>).

class Cone : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Cone();
  /// Constructs a Cone from its attributes.
  Cone(SFFloat bottomRadius, 
       SFFloat height, 
       SFBool side, 
       SFBool bottom);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Cone();
  
  /// Sets the BottomRadius attribute.
  void setBottomRadius(SFFloat bottomRadius);
  /// Sets the Height attribute.
  void setHeight(SFFloat height);
  /// Sets the Side attribute.
  void setSide(SFBool side);
  /// Sets the Bottom attribute.
  void setBottom(SFBool bottom);
  
  /// Gets the BottomRadius attribute.
  inline SFFloat getBottomRadius() const {return _bottomRadius;};
  /// Gets the Height attribute.
  inline SFFloat getHeight() const {return _height;};
  /// Gets the Side attribute.
  inline SFBool getSide() const {return _side;};
  /// Gets the Bottom attribute.
  inline SFBool getBottom() const {return _bottom;};

  /// Loads the attributes from a X3DFileElement e.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.	   
  Cone(const Cone &C);  

private:  
  /// Bottom radius attribute.
  SFFloat _bottomRadius;
  /// Height attribute.
  SFFloat _height;
  /// Side attribute.
  SFBool _side;
  /// Bottom attribute.
  SFBool _bottom;
};

}
}

#endif
