/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            GL_Cone.h                                       //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLCONE_H
#define GLCONE_H

#include "GL_X3DGeometry3DNode.h"

namespace X3DTK {
namespace GL {

class ConeDrawArray;

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode
/// Cone.

class Cone : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Cone();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Cone();

  /// Sets the bottom radius attribute.
  void setBottomRadius(SFFloat bottomRadius);
  /// Sets the height attribute.
  void setHeight(SFFloat height);
  /// Sets the side attribute.
  void setSide(SFBool side);
  /// Sets the bottom attribute.
  void setBottom(SFBool bottom);
  /// Sets the cone array.
  void setConeArray(ConeDrawArray *coneArray);

  /// Gets the bottom radius attribute.
  inline SFFloat getBottomRadius() const {return _bottomRadius;};
  /// Gets the height attribute.
  inline SFFloat getHeight() const {return _height;};
  /// Gets the side attribute.
  inline SFBool getSide() const {return _side;};
  /// Gets the bottom attribute.
  inline SFBool getBottom() const {return _bottom;};
  /// Gets the cone array.
  inline ConeDrawArray *getConeArray() const {return _coneArray;};

  /// Updates the attributes of the node.
  virtual void update();
  /// Renders the node in an openGL context.
  virtual void render() const;  
  
protected:
  /// Copy constructor.	   
  Cone(const Cone &C);
  
  /// Bottom radius attribute.
  SFFloat _bottomRadius;
  /// Height attribute.
  SFFloat _height;
  /// Side attribute.
  SFBool _side;
  /// Bottom attribute.
  SFBool _bottom;
  /// Cone array.
  ConeDrawArray *_coneArray;
};

}
}

#endif
