/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            GL_Cylinder.h                                   //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLCYLINDER_H
#define GLCYLINDER_H

#include "GL_X3DGeometry3DNode.h"

namespace X3DTK {
namespace GL {

class CylinderDrawArray;

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// Cylinder.

class Cylinder : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Cylinder();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Cylinder();
  
  /// Radius attribute.
  void setRadius(SFFloat radius);
  /// Height attribute.
  void setHeight(SFFloat height);
  /// Side attribute.
  void setSide(SFBool side);
  /// Bottom attribute.
  void setBottom(SFBool bottom);
  /// Top attribute.
  void setTop(SFBool top);
  /// Cylinder array.
  void setCylinderArray(CylinderDrawArray *cylinderArray);
  
  /// Radius attribute.
  inline SFFloat getRadius() const {return _radius;};
  /// Height attribute.
  inline SFFloat getHeight() const {return _height;};
  /// Side attribute.
  inline SFBool getSide() const {return _side;};
  /// Bottom attribute.
  inline SFBool getBottom() const {return _bottom;};
  /// Top attribute.
  inline SFBool getTop() const {return _top;};
  /// Cylinder array.
  inline CylinderDrawArray *getCylinderArray() const {return _cylinderArray;};
  
  /// Updates the attributes of the node.
  virtual void update();
  /// Renders the node in an openGL context.
  virtual void render() const; 

protected: 
  /// Copy constructor.
  Cylinder(const Cylinder &C);

private:  
  /// Radius attribute.
  SFFloat _radius;
  /// Height attribute.
  SFFloat _height;
  /// Side attribute.
  SFBool _side;
  /// Bottom attribute.
  SFBool _bottom;
  /// Top attribute.
  SFBool _top;
  /// Cylinder array.
  CylinderDrawArray *_cylinderArray;
};

}
}

#endif
