/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            GraphTraversal.h                                //
////////////////////////////////////////////////////////////////////////////////

#ifndef GRAPHTRAVERSAL_H
#define GRAPHTRAVERSAL_H

#include "X3DTypes.h"

namespace X3DTK {

class Walker;
class Visitor; 
class X3DComponentWalker;
class X3DComponentVisitor;

/// Class defining the concept of tree traversal algorithm.

class GraphTraversal
{
public:
  /// Constructor.
  GraphTraversal();
  /// Destructor.  
  virtual ~GraphTraversal();
  
  /// Sets a component walker. 
  void setComponentWalker(X3DComponentWalker *component);
  /// Sets a component visitor. 
  void setComponentVisitor(X3DComponentVisitor *component);
  /// Deletes the components when the object is deleted.
  void setAutoDeleteComponents(bool value);
  /// Traverses the scene.
  void traverse(SFAbstractNode N) const;
  
  /// Joins the content of D1 and D2.
  friend GraphTraversal *joinGraphTraversals(GraphTraversal *A1, GraphTraversal *A2);
  /// Merges the content of D1 and D2 and deletes D1 and D2.  
  friend GraphTraversal *mergeGraphTraversals(GraphTraversal *A1, GraphTraversal *A2);
    
protected:
  /// Visitor.
  Visitor *visitor;     
  /// Walker.
  Walker *walker;
};

}

#endif
