/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            SFPoint.h                                       //
////////////////////////////////////////////////////////////////////////////////

#ifndef POINT3F_H
#define POINT3F_H

#include "SFVec3f.h"

#include <math.h>
#include <iostream>

namespace X3DTK {

class SFString;

/// Class providing an implementation of a 3D float point. 
/// SFPoint3f is a point in an affin space, which different from a vector represented
/// by the SFVec3f class.

class SFPoint3f
{
public:
   /// x coordinates.
  float x;
  /// y coordinates.
  float y;
  /// z coordinates.
  float z;

  /// Default constructor.
  SFPoint3f();
  /// Constructor with x, y and z coordinates in parameters.
  SFPoint3f(float x, float y, float z);
  /// Copy constructor.
  SFPoint3f(const SFPoint3f &p);
  /// Overloaded assignment operator.
  SFPoint3f &operator= (const SFPoint3f &v);
  /// Overloaded operator.
  SFPoint3f &operator+= (const SFVec3f &v);
  /// Overloaded operator.
  SFPoint3f &operator-= (const SFVec3f &v);
  /// Explicit constructor from a string of which format is "x y z".
  explicit SFPoint3f(const SFString &s);
  // Conversion from vector to point.
  explicit SFPoint3f(const SFVec3f &V);

   /// Overloaded operator+.
  friend SFPoint3f operator+ (const SFPoint3f &v1, const SFVec3f &v2);
  /// Overloaded operator-.
  friend SFPoint3f operator- (const SFPoint3f &v1, const SFVec3f &v2);
  /// Overloaded operator-.
  friend SFVec3f operator- (const SFPoint3f &v1, const SFPoint3f &v2);
  /// Overloaded operator*. 
  friend SFPoint3f operator* (const float a, const SFPoint3f &v);
  /// Returns the distance between the points A and B.
  friend float distance(const SFPoint3f &A, const SFPoint3f &B);
  /// Returns the pointed array.
  inline float *f_data() {return &x;};
  /// Returns the pointed array.
  inline const float *f_data() const {return &x;};
  /// origin point.
  static const SFPoint3f ORIGIN;
};

}

#endif
