/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            Visitor.h                                       //
////////////////////////////////////////////////////////////////////////////////

#ifndef VISITOR_H
#define VISITOR_H

#include "X3DAbstractNode.h"
#include "VisitingFunctions.h"
#include "EnterFunction.h"
#include "WalkOnFunction.h"
#include "LeaveFunction.h"
#include "X3DActor.h"

#include <list>

namespace X3DTK {

class Type;
class X3DComponentVisitor;

/// Concrete class defining a NodeProxy specialized in the visiting of nodes.

class Visitor : public X3DActor
{
public:  
  /// Constructor.
  Visitor();
  /// Destructor.
  virtual ~Visitor();
  
  /// Sets the X3DComponentCreator.
  void setComponentVisitor(X3DComponentVisitor *component);
  /// Enters the SFNode.
  inline void enter(SFAbstractNode N) const {_visitingArray[N->getType()->getId()]->enter(N);};
  /// Walks On the SFNode.
  inline bool walkOn(SFAbstractNode N, SFAbstractNode child = 0) const {return _visitingArray[N->getType()->getId()]->walkOn(N, child);};
  /// Leaves the SFNode.
  inline void leave(SFAbstractNode N) const {_visitingArray[N->getType()->getId()]->leave(N);};
  /// Empties the list of component.
  void reset();
  
  friend Visitor *joinVisitors(Visitor *N0, Visitor *N1);
  
private:  
  /// Adds a type.
  virtual void addType(const Type *type);
  /// finds the lowest functions in the derivation tree.
  EnterFunction *getEnterFunctionOf(const Type *type) const;
  WalkOnFunction *getWalkOnFunctionOf(const Type *type) const;
  LeaveFunction *getLeaveFunctionOf(const Type *type) const;
  
  //Dictionary of all the creation functions used.
  VisitingArray _visitingArray;
  std::list<X3DComponentVisitor *> _componentList;
};

}

#endif
