/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3DLoader.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DLOADER_H
#define X3DLOADER_H

#include "X3DTypes.h"

namespace X3DTK {
namespace X3D {

class Creator;
class X3DComponentCreator;
class Scene;
class X3DFileValidator;

/// Abstract class defining an interface for an X3D file loader.

class X3DLoader
{
public:
  /// Constructor.
  X3DLoader();
  /// Destructor.
  virtual ~X3DLoader() = 0;
  
  /// Sets the componentVisitor.
  void setComponentCreator(X3DComponentCreator *component);
  /// Sets the fileValidator.
  void setFileValidator(X3D::X3DFileValidator *fileValidator);  
  
  /// Loads a scene from the file. If fileValidation is true, then the file will be 
  /// verified and messages will be written if the file is invalid. Nevertheless the 
  /// content of the scene is not deleted.
  X3D::Scene *load(const char *file, bool fileValidation = true) const;
  
  /// Gets the fileValidator.
  inline X3D::X3DFileValidator *getFileValidator() const {return fileValidator;};  
  
protected:
  /// Creator.
  Creator *creator;
  /// FileValidator.
  X3D::X3DFileValidator *fileValidator;  
  
  /// Loads a scene from the file. This method has to be redefined.
  virtual X3D::Scene *loadFile(const char *file, bool fileValidation) const = 0;
};

}
}

#endif
