/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            GL_X3DNode.h                                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DGLNODE_H
#define X3DGLNODE_H

#include "X3DAbstractNode.h"
#include "X3DNode.h"
#include "GLSceneGraphTypes.h"

namespace X3DTK {
namespace GL {

/// Base class of the Core component. A set of GL::X3DNode represents a scene graph
/// related with the X3D scene graph. Each X3DNode able to be drawn directly has 
/// its GL::X3DNode correspondence. For example X3D::IndexedFaceSet is related with 
/// GL::IndexedFaceSet, whereas Coordinate has no GL::X3DNode equivalent. 

class X3DNode : public X3DAbstractNode
{
public:
  /// Constructor.
  X3DNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DNode();
  
  /// Sets the x3dReference.
  void setX3DReference(X3D::SFNode N);
  /// Returns the X3DNode equivalent.
  inline X3D::SFNode getX3DReference() const {return x3dReference;};
  
  /// Updates the attributes of the node.
  virtual void update();
  /// Renders the node in an openGL context.
  virtual void render() const;  
  
protected:
  /// Copy constructor.
  X3DNode(const X3DNode &N);
  
  /// Pointer to the X3DNode equivalent.
  X3D::SFNode x3dReference;
};

}
}

#endif
