/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            SphereDrawArray.h                               //
////////////////////////////////////////////////////////////////////////////////

#ifndef SPHEREARRAY_H
#define SPHEREARRAY_H

#include "GLSceneGraphTypes.h"

#include <vector>

namespace X3DTK {
namespace GL {

/// Class providing the arrays required by the opengl command drawElements() to draw
/// a Sphere.

class SphereDrawArray
{
public:
  /// Gets the CylinderDrawArray of section section.
  static SphereDrawArray *getInstanceOfSection(unsigned int section);
  /// Removes the CylinderDrawArray of section section.
  void removeInstance();
  
  
  /// Gets the number of vertices of the sphere.
  unsigned int getSphereSize() const;
  /// Gets the address of the first element of the vertex array.
  const void *getSphereVertexArrayAddress() const;
  /// Gets the address of the first index of the index array. 
  const unsigned int *getSphereIndexArrayAddress() const;
  
private:
  unsigned int _section;

  /// Constructs the arrays for a sphere of sub_theta subdivisions for the theta angle 
  /// and sub_phi subdivisions for the phi angle
  SphereDrawArray(unsigned int section);

  typedef struct 
  {
    unsigned int count;
    SphereDrawArray *ref;
  } data;
  
  typedef std::list<std::pair<unsigned int, data> > refList;

  static refList _refList;

  std::vector<N3F_V3F> _sphereVertexArray;
  std::vector<unsigned int> _sphereIndexArray;
};

}
}

#endif
