/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            GL_X3DComposedGeometryNode.h                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DGLCOMPOSEDGEOMETRYNODE_H
#define X3DGLCOMPOSEDGEOMETRYNODE_H

#include "GL_X3DGeometry3DNode.h"

namespace X3DTK {
namespace GL {

/// Class providing an interface for the equivalent of the X3DComposedGeometryNode in the X3DGL tree.

class X3DComposedGeometryNode : public X3DGeometry3DNode
{
public:
  /// Constructor.
  X3DComposedGeometryNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DComposedGeometryNode();

  /// Sets the ccw attribute.
  void setCcw(SFBool ccw);
  /// Sets the solid attribute.
  void setSolid(SFBool solid);
  /// Sets the vertices duplicated attribute.
  void setVerticesDuplicated(SFBool verticesDuplicated);
  /// Sets the color attribute.
  void setColor(SFBool color);
  /// Sets the texCoord attribute.
  void setTexCoord(SFBool texCoord);
  /// Sets the normalPerVertex attribute.  
  void setNormalPerVertex(SFBool normalPerVertex);
  /// Sets the vertex format.
  void setVertexFormat(GLenum format);

  /// gets the ccw attribute.
  inline SFBool getCcw() const {return _ccw;};
  /// Gets the solid attribute.
  inline SFBool getSolid() const {return _solid;};
  /// Gets the vertices duplicated attribute.
  inline SFBool getVerticesDuplicated() const {return _verticesDuplicated;};
  /// Gets the color attribute.
  inline SFBool getColor() const {return _color;};
  /// Gets the texCoord attribute.
  inline SFBool getTexCoord() const {return _texCoord;};
  /// Gets the normalPerVertex attribute.  
  inline SFBool getNormalPerVertex() {return _normalPerVertex;};
  /// Gets the vertex format.
  inline GLenum getVertexFormat() const {return _format;};
  
protected:
  /// Copy constructor.
  X3DComposedGeometryNode(const X3DComposedGeometryNode &N);

private:  
  /// Ccw attribute.
  SFBool _ccw;
  /// Solid attribute.
  SFBool _solid;
  /// Vertices duplicated attribute.
  SFBool _verticesDuplicated;
  /// Color attribute.
  SFBool _color;
  /// TexCoord attribute.
  SFBool _texCoord;
  /// NormalPerVertex.
  SFBool _normalPerVertex;
  /// vertex format.
  GLenum _format;
};

}
}

#endif
