#include "GL_Appearance.h"
#include "GL_X3DMaterialNode.h"
#include "GL_X3DTextureNode.h"
#include "GL_X3DTextureTransformNode.h"
#include "GL_LineProperties.h"
#include "GL_FillProperties.h"

namespace X3DTK {
namespace GL {

Appearance::Appearance()
: X3DAppearanceNode(), _material(0), _texture(0), _textureTransform(0), _lineProperties(0), _fillProperties(0)
{
  defineTypeName("Appearance", "Shape");
}

Appearance::Appearance(const Appearance &A)
: X3DAppearanceNode(A), _material(A._material), _texture(A._texture), _textureTransform(A._textureTransform), _lineProperties(A._lineProperties), _fillProperties(A._fillProperties)
{
}

SFNode Appearance::clone() const
{
  return new Appearance(*this);
}

Appearance::~Appearance()
{
  removeParentFromChild(this, _material);
  removeParentFromChild(this, _texture);
  removeParentFromChild(this, _textureTransform);
  removeParentFromChild(this, _lineProperties);
  removeParentFromChild(this, _fillProperties);
}

void Appearance::setMaterial(const X3DMaterialNode *material) 
{
  removeParentFromChild(this, _material);
  _material = (X3DMaterialNode *)material;
  addParentToChild(this, _material);
}
  
void Appearance::setTexture(const X3DTextureNode *texture) 
{
  removeParentFromChild(this, _texture);
  _texture = (X3DTextureNode *)texture;
  addParentToChild(this, _texture);
}

void Appearance::setTextureTransform(const X3DTextureTransformNode *textureTransform) 
{
  removeParentFromChild(this, _textureTransform);
  _textureTransform = (X3DTextureTransformNode *)textureTransform;
  addParentToChild(this, _textureTransform);
}

void Appearance::setLineProperties(const LineProperties *lineProperties) 
{
  removeParentFromChild(this, _lineProperties);
  _lineProperties = (LineProperties *)lineProperties;
  addParentToChild(this, _lineProperties);
}

void Appearance::setFillProperties(const FillProperties *fillProperties) 
{
  removeParentFromChild(this, _fillProperties);
  _fillProperties = (FillProperties *)fillProperties;
  addParentToChild(this, _fillProperties);
}

bool Appearance::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DMaterialNode *>(N) != 0)
  {
    if (_material != 0)
      return false;
      
    _material = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;
  }
  
  if (dynamic_cast<X3DTextureNode *>(N) != 0)
  {
    if (_texture != 0)
      return false;
      
    _texture = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;    
  }
  
  if (dynamic_cast<X3DTextureTransformNode *>(N) != 0)
  {
    if (_textureTransform != 0)
      return false;
      
    _textureTransform = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;    
  }
  
  if (dynamic_cast<LineProperties *>(N) != 0)
  {
    if (_lineProperties != 0)
      return false;
      
    _lineProperties = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;    
  }
  
  if (dynamic_cast<FillProperties *>(N) != 0)
  {
    if (_fillProperties != 0)
      return false;
      
    _fillProperties = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;    
  }
  
  return false;
}

bool Appearance::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DMaterialNode *>(N) != 0)
  {
    removeParentFromChild(this, N);
    _material = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;
  }
  
  if (dynamic_cast<X3DTextureNode *>(N) != 0)
  {
    removeParentFromChild(this, N);
    _texture = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;    
  }
  
  if (dynamic_cast<X3DTextureTransformNode *>(N) != 0)
  {
    removeParentFromChild(this, N);
    _textureTransform = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;    
  }
  
  if (dynamic_cast<LineProperties *>(N) != 0)
  {
    removeParentFromChild(this, N);
    _lineProperties = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;    
  }
  
  if (dynamic_cast<FillProperties *>(N) != 0)
  {
    removeParentFromChild(this, N);
    _fillProperties = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;    
  }
  
  return false;
}

bool Appearance::removeChild(const SFAbstractNode &N)
{
  if (_material == N)
  {
    removeParentFromChild(this, N);
    _material = 0;
    return true;
  }
  
  if (_texture == N)
  {
    removeParentFromChild(this, N);
    _texture = 0;
    return true; 
  }
  
  if (_textureTransform == N)
  {
    removeParentFromChild(this, N);
    _textureTransform = 0;
    return true;  
  }
  
  if (_lineProperties == N)
  {
    removeParentFromChild(this, N);
    _lineProperties = 0;
    return true;
  }
  
  if (_fillProperties == N)
  { 
    removeParentFromChild(this, N);
    _fillProperties = 0;
    return true; 
  }
  
  return false;
}

}
}
