/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            GL_Material.h                                   //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLMATERIAL_H
#define GLMATERIAL_H

#include "GL_X3DMaterialNode.h"

#include <vector>

namespace X3DTK {
namespace GL {

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// Material.

class Material : public X3DMaterialNode
{
public:
  /// Constructor.
  Material();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Material();
  
  /// Sets the diffuse color attribute.
  void setDiffuseColor(const MFFloat &diffuseColor);
  /// Sets the ambient color attribute.
  void setAmbientColor(const MFFloat &ambientColor);
  /// Sets the emissisve color attribute.
  void setEmissiveColor(const MFFloat &emissiveColor);
  /// Sets the shininess attribute.
  void setShininess(SFFloat shininess);
  /// Sets the specular color attribute.
  void setSpecularColor(const MFFloat &specularColor);

  /// Gets the diffuse color attribute.
  inline const MFFloat &getDiffuseColor() const {return _diffuseColor;};
  /// Gets the ambient color attribute.
  inline const MFFloat &getAmbientColor() const {return _ambientColor;};
  /// Gets the emissisve color attribute.
  inline const MFFloat &getEmissiveColor() const {return _emissiveColor;};
  /// Gets the shininess attribute.
  inline SFFloat getShininess() const {return _shininess;};
  /// Gets the specular color attribute.
  inline const MFFloat &getSpecularColor() const {return _specularColor;};

  /// Updates the attributes of the node.
  virtual void update();
  /// Renders the node in an openGL context.
  virtual void render() const;  

protected:    
  /// Copy constructor.	   
  Material(const Material &M);

private:  
  /// Diffuse color attribute.
  MFFloat _diffuseColor;
  /// Ambient color attribute.
  MFFloat _ambientColor;
  /// Emissisve color attribute.
  MFFloat _emissiveColor;
  /// Shininess attribute.
  SFFloat _shininess;
  /// Specular color attribute.
  MFFloat _specularColor;
};

}
}

#endif
