#include "GL_X3DShapeNode.h"
#include "GL_X3DGeometryNode.h"
#include "GL_X3DAppearanceNode.h"

namespace X3DTK {
namespace GL {

X3DShapeNode::X3DShapeNode()
: X3DChildNode(), _geometry(0), _appearance(0)
{
  defineTypeName("X3DShapeNode", "Shape");
}

X3DShapeNode::X3DShapeNode(const X3DShapeNode &N)
: X3DChildNode(N), _geometry(N._geometry), _appearance(N._appearance)
{
}

SFNode X3DShapeNode::clone() const
{
  return 0;
}

X3DShapeNode::~X3DShapeNode()
{
}

void X3DShapeNode::setGeometry(const X3DGeometryNode *geometry)
{
  _geometry = (X3DGeometryNode *)geometry;
}

void X3DShapeNode::setAppearance(const X3DAppearanceNode *appearance)
{
  _appearance = (X3DAppearanceNode *)appearance;
}

bool X3DShapeNode::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DGeometryNode *>(N) != 0)
  {
    if (_geometry != 0)
      return false;
      
    _geometry = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;
  }
  
  if (dynamic_cast<X3DAppearanceNode *>(N) != 0)
  {
    if (_appearance != 0)
      return false;
      
    _appearance = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;    
  }
  
  return false;
}

bool X3DShapeNode::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DGeometryNode *>(N) != 0)
  {
    removeParentFromChild(this, N);
    _geometry = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;
  }
  
  if (dynamic_cast<X3DAppearanceNode *>(N) != 0)
  {
    removeParentFromChild(this, N);
    _appearance = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;    
  }
  
  return false;
}

bool X3DShapeNode::removeChild(const SFAbstractNode &N)
{
  if (_geometry == N)
  {
    removeParentFromChild(this, N);
    _geometry = 0;
    return true;
  } 
   
  if (_appearance == N)
  {
    removeParentFromChild(this, N);
    _appearance = 0;
    return true;
  } 
  
  return false;
}

}
}
