#include "GL_ImageTexture.h"
#include "ImageTexture.h"
#include "Image.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace GL {

ImageTexture::ImageTexture()
: X3DTexture2DNode(), _texName(0)
{
  defineTypeName("ImageTexture", "Texturing");
  
  _textureTransform[0] = 1.0f;
  _textureTransform[1] = 0.0f;
  _textureTransform[2] = 0.0f;
  _textureTransform[3] = 0.0f;
  
  _textureTransform[4] = 0.0f;
  _textureTransform[5] = 1.0f;
  _textureTransform[6] = 0.0f;
  _textureTransform[7] = 0.0f;

  _textureTransform[8] = 0.0f;
  _textureTransform[9] = 0.0f;
  _textureTransform[10] = 1.0f;
  _textureTransform[11] = 0.0f;

  _textureTransform[12] = 0.0f;
  _textureTransform[13] = 0.0f;
  _textureTransform[14] = 0.0f;
  _textureTransform[15] = 1.0f;
}

ImageTexture::ImageTexture(const ImageTexture &I)
: X3DTexture2DNode(I), _texName(I._texName)
{
  _textureTransform[0] = I._textureTransform[0];
  _textureTransform[1] = I._textureTransform[1];
  _textureTransform[2] = I._textureTransform[2];
  _textureTransform[3] = I._textureTransform[3];
  
  _textureTransform[4] = I._textureTransform[4];
  _textureTransform[5] = I._textureTransform[5];
  _textureTransform[6] = I._textureTransform[6];
  _textureTransform[7] = I._textureTransform[7];

  _textureTransform[8] = I._textureTransform[8];
  _textureTransform[9] = I._textureTransform[9];
  _textureTransform[10] = I._textureTransform[10];
  _textureTransform[11] = I._textureTransform[11];

  _textureTransform[12] = I._textureTransform[12];
  _textureTransform[13] = I._textureTransform[13];
  _textureTransform[14] = I._textureTransform[14];
  _textureTransform[15] = I._textureTransform[15];  
}

SFNode ImageTexture::clone() const
{
  return new ImageTexture(*this);
}

ImageTexture::~ImageTexture()
{
  glDeleteTextures(1, &_texName);
}

void ImageTexture::setTexName(GLuint texName)
{
  _texName = texName;
}

void ImageTexture::update()
{
  if (x3dReference == 0)
    return;
    
  X3D::ImageTexture *I = static_cast<X3D::ImageTexture *>(x3dReference);
    
  MFString url = I->getUrl();
  if (url.empty())
    return;
  
  SFString firstUrl(SFString(url.front()));
  const char *file = firstUrl;
  
  Image texImage;
  if (!texImage.load(file))
  {
    cerr << "cannot load the image " << file << "!" << endl;
    return;
  }  
  
  getTextureTransform();
    
  texImage.resizeGL(_textureTransform);

  // GL commands
  glGenTextures(1, &_texName);
  
  glBindTexture(GL_TEXTURE_2D, _texName);
  
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

  if (I->getRepeatS())
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  else
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    
  if (I->getRepeatT())
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  else
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, texImage.getWidth(), texImage.getHeight(), 0, texImage.getGLType(), GL_UNSIGNED_BYTE, texImage.getData());
}

void ImageTexture::render() const
{
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  glBindTexture(GL_TEXTURE_2D, _texName);
  
  glMatrixMode(GL_TEXTURE);
  glLoadMatrixf(_textureTransform);
}

}
}
