/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            DirectionalLight.h                              //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef DIRECTIONALLIGHT
#define DIRECTIONALLIGHT

#include "X3DTypes.h"
#include "X3DLightNode.h"

namespace X3DTK {
namespace X3D {

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/lighting.html">
/// Lighting component</a> defining an DirectionalLight
/// (<a href="../../doc/X3DSpecs/Part01/components/lighting.html#DirectionalLight">
/// documentation</a>).

class DirectionalLight : public X3DLightNode
{
public:
  /// Constructor.
  DirectionalLight();
  /// Constructs a DirectionalLight from its attributes.
  DirectionalLight(SFFloat       ambientIntensity,
                   const SFColor &color,
                   const SFVec3f &direction,
                   SFFloat       intensity,
                   SFBool        on);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~DirectionalLight();

  /// Sets the direction attribute.
  void setDirection(const SFVec3f &direction);

  /// Gets the direction attribute.
  inline const SFVec3f &getDirection() const {return _direction;};

  /// Loads the attributes from a X3DFileElement.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.
  DirectionalLight(const DirectionalLight &N);

private:
  /// Direction vector of the illumination emanating from the light source.
  SFVec3f _direction;
};

}
}

#endif
