#include "Appearance.h"
#include "X3DMaterialNode.h"
#include "X3DTextureNode.h"
#include "X3DTextureTransformNode.h"
#include "LineProperties.h"
#include "FillProperties.h"

namespace X3DTK {
namespace X3D {

Appearance::Appearance()
: X3DAppearanceNode(), _material(0), _texture(0), _textureTransform(0), _lineProperties(0), _fillProperties(0)
{
  defineTypeName("Appearance", "Shape");
}

Appearance::Appearance(const X3DMaterialNode *material,  const X3DTextureNode *texture, const X3DTextureTransformNode *textureTransform, const LineProperties *lineProperties, const FillProperties *fillProperties)
: X3DAppearanceNode(), _material((SFNode)material), _texture((SFNode)texture), _textureTransform((SFNode)textureTransform), _lineProperties((SFNode)lineProperties), _fillProperties((SFNode)fillProperties)
{
  defineTypeName("Appearance", "Shape");
}

Appearance::Appearance(const Appearance &A)
: X3DAppearanceNode(A), _material(A._material), _texture(A._texture), _textureTransform(A._textureTransform), _lineProperties(A._lineProperties), _fillProperties(A._fillProperties)
{
}

SFNode Appearance::clone() const
{
  return new Appearance(*this);
}

Appearance::~Appearance()
{
  removeParentFromChild(this, _material);
  removeParentFromChild(this, _texture);
  removeParentFromChild(this, _textureTransform);
  removeParentFromChild(this, _lineProperties);
  removeParentFromChild(this, _fillProperties);
}

void Appearance::setMaterial(const X3DMaterialNode *material)
{
  removeParentFromChild(this, _material);
  _material = (SFNode)material;
  addParentToChild(this, _material);
}

void Appearance::setTexture(const X3DTextureNode *texture)
{
  removeParentFromChild(this, _texture);
  _texture = (SFNode)texture;
  addParentToChild(this, _texture);
}

void Appearance::setTextureTransform(const X3DTextureTransformNode *textureTransform)
{
  removeParentFromChild(this, _textureTransform);
  _textureTransform = (SFNode)textureTransform;
  addParentToChild(this, _textureTransform);
}

void Appearance::setLineProperties(const LineProperties *lineProperties)
{
  removeParentFromChild(this, _lineProperties);
  _lineProperties = (SFNode)lineProperties;
  addParentToChild(this, _lineProperties);
}

void Appearance::setFillProperties(const FillProperties *fillProperties)
{
  removeParentFromChild(this, _fillProperties);
  _fillProperties = (SFNode)fillProperties;
  addParentToChild(this, _fillProperties);
}

bool Appearance::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DMaterialNode *>(N) != 0)
  {
    if (_material != 0)
      return false;
      
    setMaterial(static_cast<X3DMaterialNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DTextureNode *>(N) != 0)
  {
    if (_texture != 0)
      return false;
      
    setTexture(static_cast<X3DTextureNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<X3DTextureTransformNode *>(N) != 0)
  {
    if (_textureTransform != 0)
      return false;
      
    setTextureTransform(static_cast<X3DTextureTransformNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<LineProperties *>(N) != 0)
  {
    if (_lineProperties != 0)
      return false;
      
    setLineProperties(static_cast<LineProperties *>(N));
    return true;    
  }
  
  if (dynamic_cast<FillProperties *>(N) != 0)
  {
    if (_fillProperties != 0)
      return false;
      
    setFillProperties(static_cast<FillProperties *>(N));
    return true;    
  }
  
  return false;
}

bool Appearance::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DMaterialNode *>(N) != 0)
  {
    setMaterial(static_cast<X3DMaterialNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DTextureNode *>(N) != 0)
  {
    setTexture(static_cast<X3DTextureNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<X3DTextureTransformNode *>(N) != 0)
  {
    setTextureTransform(static_cast<X3DTextureTransformNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<LineProperties *>(N) != 0)
  {
    setLineProperties(static_cast<LineProperties *>(N));
    return true;    
  }
  
  if (dynamic_cast<FillProperties *>(N) != 0)
  {
    setFillProperties(static_cast<FillProperties *>(N));
    return true;    
  }
  
  return false;
}

bool Appearance::removeChild(const SFAbstractNode &N)
{
  if (_material == N)
  {
    setMaterial(0);
    return true;
  }
  
  if (_texture == N)
  {
    setTexture(0);
    return true; 
  }
  
  if (_textureTransform == N)
  {
    setTextureTransform(0);    
    return true;  
  }
  
  if (_lineProperties == N)
  {
    setLineProperties(0);
    return true;
  }
  
  if (_fillProperties == N)
  {
    setFillProperties(0);
    return true; 
  }
  
  return false;
}

}
}
