#include "FillProperties.h"

namespace X3DTK {
namespace X3D {

FillProperties::FillProperties()
: X3DAppearanceChildNode(), _fillStyle(SFString("")), _hatchStyle(1), _hatchColor(SFColor(1.0f, 1.0f, 1.0f))
{
  defineTypeName("FillProperties", "Shape");
}

FillProperties::FillProperties(SFString fillStyle, SFInt32 hatchStyle, SFColor hatchColor)
: X3DAppearanceChildNode(), _fillStyle(fillStyle), _hatchStyle(hatchStyle), _hatchColor(hatchColor)
{
  defineTypeName("FillProperties", "Shape");
}

FillProperties::FillProperties(const FillProperties &F)
: X3DAppearanceChildNode(F), _fillStyle(F._fillStyle), _hatchStyle(F._hatchStyle), _hatchColor(F._hatchColor)
{
}

SFNode FillProperties::clone() const
{
  return new FillProperties(*this);
}

FillProperties::~FillProperties()
{
}

void FillProperties::setFillStyle(SFString fillStyle)
{
  _fillStyle = fillStyle;
}

void FillProperties::setHatchStyle(SFInt32 hatchStyle)
{
  _hatchStyle = hatchStyle;
}

void FillProperties::setHatchColor(SFColor hatchColor)
{
  _hatchColor = hatchColor;
}

void FillProperties::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("fillStyle");
  if (index != -1)
    _fillStyle = element->getAttribute(index);
  
  index = element->getIndexAttribute("hatchStyle");
  if (index != -1)
    _hatchStyle = element->getAttribute(index).toInt();  
  
  index = element->getIndexAttribute("hatchColor");
  if (index != -1)
    _hatchColor = SFColor(element->getAttribute(index));    
}

SFString FillProperties::write() const
{
  SFString attr;
  if (_fillStyle != "NONE")
    attr += " fillStyle = \"" + _fillStyle + "\"";
  if (_hatchStyle != 1)
    attr += " hatchStyle = \"" + toSFString(_hatchStyle) + "\"";  
  if (_hatchColor != SFColor(1.0f, 1.0f, 1.0f))
    attr += " hatchColor = \"" + toSFString(_hatchColor) + "\"";
      
  return attr;
}

}
}
