#include "X3DShapeNode.h"
#include "X3DGeometryNode.h"
#include "X3DAppearanceNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

X3DShapeNode::X3DShapeNode()
: X3DChildNode(), _geometry(0), _appearance(0)
{
  defineTypeName("X3DShapeNode", "Shape");
}

X3DShapeNode::X3DShapeNode(const X3DGeometryNode *geometry,  const X3DAppearanceNode *appearance)
: X3DChildNode(), _geometry((SFNode)geometry), _appearance((SFNode)appearance)
{
  defineTypeName("X3DShapeNode", "Shape");
}

X3DShapeNode::X3DShapeNode(const X3DShapeNode &N)
: X3DChildNode(N), _geometry(N._geometry), _appearance(N._appearance)
{
}

SFNode X3DShapeNode::clone() const
{
  return 0;
}

X3DShapeNode::~X3DShapeNode()
{
  removeParentFromChild(this, _geometry);
  removeParentFromChild(this, _appearance);
}

void X3DShapeNode::setGeometry(const X3DGeometryNode *geometry)
{
  removeParentFromChild(this, _geometry);
  _geometry = (SFNode)geometry;
  addParentToChild(this, _geometry);
}

void X3DShapeNode::setAppearance(const X3DAppearanceNode *appearance)
{
  removeParentFromChild(this, _appearance);
  _appearance = (SFNode)appearance;
  addParentToChild(this, _appearance);
}

bool X3DShapeNode::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DGeometryNode *>(N) != 0)
  {
    if (_geometry != 0)
      return false;
      
    setGeometry(static_cast<X3DGeometryNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DAppearanceNode *>(N) != 0)
  {
    if (_appearance != 0)
      return false;
      
    setAppearance(static_cast<X3DAppearanceNode *>(N));
    return true;    
  }
  
  return false;
}

bool X3DShapeNode::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DGeometryNode *>(N) != 0)
  {
    setGeometry(static_cast<X3DGeometryNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DAppearanceNode *>(N) != 0)
  {
    setAppearance(static_cast<X3DAppearanceNode *>(N));
    return true;    
  }
  
  return false;
}

bool X3DShapeNode::removeChild(const SFAbstractNode &N)
{
  if (_geometry == N)
  {
    setGeometry(0);
    return true;
  } 
   
  if (_appearance == N)
  {
    setAppearance(0);
    return true;
  } 
  
  return false;
}

}
}
