#include "BBoxUpdaterGroupingVisitor.h"
#include "StaticGroup.h"
#include "X3DGroupingNode.h"
#include "Group.h"
#include "Transform.h"
#include "Switch.h"
#include "BBox.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

BBoxUpdaterGroupingVisitor::BBoxUpdaterGroupingVisitor()
: GroupingVisitor()
{
  // WalkOn functions.
  defineNewWalkOnFunction(&BBoxUpdaterGroupingVisitor::walkOnStaticGroup);
  defineNewWalkOnFunction(&BBoxUpdaterGroupingVisitor::walkOnX3DGroupingNode);

  // Leave functions.
  defineNewLeaveFunction(&BBoxUpdaterGroupingVisitor::leaveStaticGroup);
  defineNewLeaveFunction(&BBoxUpdaterGroupingVisitor::leaveGroup);
  defineNewLeaveFunction(&BBoxUpdaterGroupingVisitor::leaveTransform);
  defineNewLeaveFunction(&BBoxUpdaterGroupingVisitor::leaveSwitch);
  
  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<BBoxUpdaterGlobalVariables>();
}

BBoxUpdaterGroupingVisitor::~BBoxUpdaterGroupingVisitor()
{
}

bool BBoxUpdaterGroupingVisitor::walkOnStaticGroup(StaticGroup *, SFAbstractNode) const
{
  return true;
}

bool BBoxUpdaterGroupingVisitor::walkOnX3DGroupingNode(X3DGroupingNode *, SFAbstractNode) const
{
  return true;
}

void BBoxUpdaterGroupingVisitor::leaveStaticGroup(StaticGroup *S) const
{
  BBox *BB = globalVariables->getBBox(S);
  if (BB == 0)
  {
    if (!globalVariables->getStaticProcessing())
    {
      BB = new BBox(S->getBBoxCenter(), S->getBBoxSize());
      globalVariables->addBBox(S, BB);
    }
    else
    {  
      
      MFNode children = S->getChildren();
      for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
      {
        X3DBoundedObject *BO = dynamic_cast<X3DBoundedObject *>(*it);
        if (BO != 0)
          globalVariables->addBBoxToMergeList(BBox(BO->getBBoxCenter(), BO->getBBoxSize()));
      }
      
      BB = new BBox(globalVariables->mergeBBox()); 
      globalVariables->addBBox(S, BB);
    
      //updating the BBox attributes.
      S->setBBoxCenter(BB->getCenter());
      S->setBBoxSize(BB->getSize());
    }
  }
}

void BBoxUpdaterGroupingVisitor::leaveGroup(Group *G) const
{
  BBox *BB = globalVariables->getBBox(G);
  if (BB == 0)
  {
    MFNode children = G->getChildren();
    for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
    {
      X3DBoundedObject *BO = dynamic_cast<X3DBoundedObject *>(*it);
      if (BO != 0)
        globalVariables->addBBoxToMergeList(BBox(BO->getBBoxCenter(), BO->getBBoxSize()));
    }
    
    BB = new BBox(globalVariables->mergeBBox()); 
    globalVariables->addBBox(G, BB);
    
    //updating the BBox attributes.
    G->setBBoxCenter(BB->getCenter());
    G->setBBoxSize(BB->getSize());
  }
}

void BBoxUpdaterGroupingVisitor::leaveTransform(Transform *T) const
{
  BBox *BB = globalVariables->getBBox(T);
  if (BB == 0)
  {
    MFNode children = T->getChildren();
    for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
    {
      X3DBoundedObject *BO = dynamic_cast<X3DBoundedObject *>(*it);
      if (BO != 0)
        globalVariables->addBBoxToMergeList(BBox(BO->getBBoxCenter(), BO->getBBoxSize()));
    }
    
    BB = new BBox(globalVariables->mergeBBox()); 
    
    //change the coordinates of the BBox in the coordinates of the parent node
    SFMatrix34f transform(T->getTranslation(), T->getCenter(), T->getRotation(), T->getScaleOrientation(), T->getScale());
    
    //comparing the 6 transformed points of the Bounding Box
    SFPoint3f Corner[8];
    Corner[0] = transform*SFPoint3f(BB->getCenter().x - 0.5f*BB->getSize().x, BB->getCenter().y - 0.5f*BB->getSize().y, BB->getCenter().z - 0.5f*BB->getSize().z);
    Corner[1] = transform*SFPoint3f(BB->getCenter().x - 0.5f*BB->getSize().x, BB->getCenter().y - 0.5f*BB->getSize().y, BB->getCenter().z + 0.5f*BB->getSize().z);
    Corner[2] = transform*SFPoint3f(BB->getCenter().x - 0.5f*BB->getSize().x, BB->getCenter().y + 0.5f*BB->getSize().y, BB->getCenter().z - 0.5f*BB->getSize().z);
    Corner[3] = transform*SFPoint3f(BB->getCenter().x - 0.5f*BB->getSize().x, BB->getCenter().y + 0.5f*BB->getSize().y, BB->getCenter().z + 0.5f*BB->getSize().z);
    Corner[4] = transform*SFPoint3f(BB->getCenter().x + 0.5f*BB->getSize().x, BB->getCenter().y - 0.5f*BB->getSize().y, BB->getCenter().z - 0.5f*BB->getSize().z);
    Corner[5] = transform*SFPoint3f(BB->getCenter().x + 0.5f*BB->getSize().x, BB->getCenter().y - 0.5f*BB->getSize().y, BB->getCenter().z + 0.5f*BB->getSize().z);
    Corner[6] = transform*SFPoint3f(BB->getCenter().x + 0.5f*BB->getSize().x, BB->getCenter().y + 0.5f*BB->getSize().y, BB->getCenter().z - 0.5f*BB->getSize().z);
    Corner[7] = transform*SFPoint3f(BB->getCenter().x + 0.5f*BB->getSize().x, BB->getCenter().y + 0.5f*BB->getSize().y, BB->getCenter().z + 0.5f*BB->getSize().z);
  
    SFPoint3f min = Corner[0];
    SFPoint3f max = Corner[0];
  
    for (unsigned int i = 0; i < 8; ++i)
    {
      if (Corner[i].x < min.x)
        min.x = Corner[i].x;
      if (Corner[i].y < min.y)
        min.y = Corner[i].y;
      if (Corner[i].z < min.z)
        min.z = Corner[i].z;
  
      if (Corner[i].x > max.x)
        max.x = Corner[i].x;
      if (Corner[i].y > max.y)
        max.y = Corner[i].y;
      if (Corner[i].z > max.z)
        max.z = Corner[i].z; 
    }    
   
    SFVec3f mid = max - min; 
    SFPoint3f center = min + 0.5f*mid;
    SFVec3f size = 2.0f*(max - center); 

    BB->setCenter(SFVec3f(center));
    BB->setSize(size);
    
    globalVariables->addBBox(T, BB);
    
    //updating the BBox attributes.
    T ->setBBoxCenter(BB->getCenter());
    T->setBBoxSize(BB->getSize());
  }
}

void BBoxUpdaterGroupingVisitor::leaveSwitch(Switch *S) const
{
  BBox *BB = globalVariables->getBBox(S);
  if (BB == 0)
  {
    MFNode children = S->getChildren();
    for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
    {
      X3DBoundedObject *BO = dynamic_cast<X3DBoundedObject *>(*it);
      if (BO != 0)
        globalVariables->addBBoxToMergeList(BBox(BO->getBBoxCenter(), BO->getBBoxSize()));
    }
    
    BB = new BBox(globalVariables->mergeBBox()); 
    globalVariables->addBBox(S, BB);
    
    //updating the BBox attributes.
    S->setBBoxCenter(BB->getCenter());
    S->setBBoxSize(BB->getSize());
  }
}

}
}
