#include "GLBuilderGroupingVisitor.h"
#include "Group.h"
#include "StaticGroup.h"
#include "Transform.h"
#include "Switch.h"
#include "GL_Group.h"
#include "GL_StaticGroup.h"
#include "GL_Transform.h"
#include "GL_Switch.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

GLBuilderGroupingVisitor::GLBuilderGroupingVisitor()
: GroupingVisitor()
{
  // Enter functions.
  defineNewEnterFunction(&GLBuilderGroupingVisitor::enterGroup);
  defineNewEnterFunction(&GLBuilderGroupingVisitor::enterStaticGroup);
  defineNewEnterFunction(&GLBuilderGroupingVisitor::enterTransform);
  defineNewEnterFunction(&GLBuilderGroupingVisitor::enterSwitch);
  
  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLBuilderGlobalVariables>();
}

GLBuilderGroupingVisitor::~GLBuilderGroupingVisitor()
{
}

void GLBuilderGroupingVisitor::enterGroup(Group *G) const
{
  GL::SFNode GG = globalVariables->getNode(G);
  if (GG == 0)
  {
    GG = new GL::Group();
    GG->setX3DReference(G);
    globalVariables->addCoupleNode(G, GG);
  }   
  globalVariables->pushNode(GG);
}

void GLBuilderGroupingVisitor::enterStaticGroup(StaticGroup *S) const
{
  GL::SFNode GS = globalVariables->getNode(S);
  if (GS == 0)
  {
    GS = new GL::StaticGroup();
    GS->setX3DReference(S);
    globalVariables->addCoupleNode(S, GS);
  }   
  globalVariables->pushNode(GS);
}

void GLBuilderGroupingVisitor::enterTransform(Transform *T) const
{
  GL::SFNode GT = globalVariables->getNode(T);
  if (GT == 0)
  {
    GT = new GL::Transform();
    GT->setX3DReference(T);
    globalVariables->addCoupleNode(T, GT);
  }   
  globalVariables->pushNode(GT);
}

void GLBuilderGroupingVisitor::enterSwitch(Switch *S) const
{
  GL::SFNode GS = globalVariables->getNode(S);
  if (GS == 0)
  {
    GS = new GL::Switch();
    GS->setX3DReference(S);
    globalVariables->addCoupleNode(S, GS);
  }   
  globalVariables->pushNode(GS);
}

}
}
