#include "MeshX3DMerger.h"
#include "DFSGraphTraversal.h"
#include "MeshX3DMergerCoreVisitor.h"
#include "MeshX3DMergerGeometry3DVisitor.h"
#include "MeshX3DMergerGroupingVisitor.h"
#include "MeshX3DMergerRenderingVisitor.h"
#include "MeshNodeListBuilder.h"
#include "MESH_Scene.h"
#include "Scene.h"

namespace X3DTK {
namespace X3D {

MeshX3DMerger::MeshX3DMerger()
{
  globalVariables = GVManager::getInstanceOf<MeshX3DMergerGlobalVariables>();
  
  graphTraversal = new DFSGraphTraversal();
  graphTraversal->setComponentVisitor(new MeshX3DMergerCoreVisitor());
  graphTraversal->setComponentVisitor(new MeshX3DMergerGeometry3DVisitor());
  graphTraversal->setComponentVisitor(new MeshX3DMergerGroupingVisitor());
  graphTraversal->setComponentVisitor(new MeshX3DMergerRenderingVisitor());
  
  listBuilder = new MESH::NodeListBuilder();
}

MeshX3DMerger::~MeshX3DMerger()
{
  GVManager::removeInstanceOf<MeshX3DMergerGlobalVariables>();

  delete graphTraversal;
  delete listBuilder;
}
  
void MeshX3DMerger::merge(MESH::Scene *MS, Scene *S)
{
  listBuilder->buildLists(MS);
  
  globalVariables->setNodeList(listBuilder->getNodeList());
  globalVariables->init();
  graphTraversal->traverse(S);
  globalVariables->finish();
}

}
}
