/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            MeshX3DMergerGlobalVariables.h                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef MESHX3DMERGERGLOBALVARIABLES_H
#define MESHX3DMERGERGLOBALVARIABLES_H

#include "GlobalVariables.h"
#include "X3DSceneGraphTypes.h"
#include "MeshSceneGraphTypes.h"
#include "MESH_X3DNode.h"
#include "X3DNode.h"
#include "MESH_Transform.h"
#include "MESH_Mesh.h"

#include <list>

namespace X3DTK {
namespace X3D {

/// GlobalVariables for the MeshX3DMerger module.

class MeshX3DMergerGlobalVariables : public GlobalVariables
{
public:
  /// Constructor.
  MeshX3DMergerGlobalVariables();
  /// Destructor.
  virtual ~MeshX3DMergerGlobalVariables();
  /// Initializes the traversal.
  void init();
  /// Finishes the traversal.
  void finish();
  
  /// Sets the Node List.
  void setNodeList(std::list<MESH::SFNode> nodeList);
  /// Pushes the parent.
  void pushParent(SFNode P);
  /// Pops the parent.
  void popParent();
  /// Sets the Exit boolean.
  void setExit(bool value);

  /// Gets the next Transform.
  MESH::Transform *getNextTransform();
  /// Gets the next Mesh.
  MESH::Mesh *getNextMesh();
  /// Gets the Exit boolean.
  inline bool getExit() const {return _exit;};

  /// Gets the parent.
  SFNode getParent() const {return _nodeStack.front();};
  
private:
  std::list<MESH::SFNode> _nodeList;
  std::list<MESH::SFNode>::const_iterator _itNode;
  MFNode _nodeStack;
  bool _exit;
};

}
}

#endif
