#include "MeshX3DMergerGroupingVisitor.h"
#include "X3DGroupingNode.h"
#include "StaticGroup.h"
#include "Group.h"
#include "Transform.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

MeshX3DMergerGroupingVisitor::MeshX3DMergerGroupingVisitor()
{
  globalVariables = GVManager::getInstanceOf<MeshX3DMergerGlobalVariables>();

  defineNewLeaveFunction(&MeshX3DMergerGroupingVisitor::leaveX3DGroupingNode);
  defineNewLeaveFunction(&MeshX3DMergerGroupingVisitor::leaveStaticGroup);
  defineNewLeaveFunction(&MeshX3DMergerGroupingVisitor::leaveGroup);
  defineNewLeaveFunction(&MeshX3DMergerGroupingVisitor::leaveTransform);
}

MeshX3DMergerGroupingVisitor::~MeshX3DMergerGroupingVisitor()
{
}

void MeshX3DMergerGroupingVisitor::leaveX3DGroupingNode(X3DGroupingNode *N) const
{
  globalVariables->popParent();

  // Transform cannot be different from identity for the other nodes than 
  // Group and Transform.
  MESH::Transform *MT = globalVariables->getNextTransform();
  if ((MT == 0) || (MT->getTransform() != SFMatrix34f::IDENTITY))
  {
    globalVariables->setExit(true);
    cerr << "MeshX3DMerger: cannot merge!" << endl;
    return;
  }
}

void MeshX3DMergerGroupingVisitor::leaveStaticGroup(StaticGroup *G) const
{
  globalVariables->popParent();

  // Transform cannot be different from identity for the other nodes than 
  // Group and Transform.
  MESH::Transform *MT = globalVariables->getNextTransform();
  if ((MT == 0) || (MT->getTransform() != SFMatrix34f::IDENTITY))
  {
    globalVariables->setExit(true);
    cerr << "MeshX3DMerger: cannot merge!" << endl;
    return;
  }
}

void MeshX3DMergerGroupingVisitor::leaveGroup(Group *G) const
{
  globalVariables->popParent();

  MESH::Transform *MT = globalVariables->getNextTransform();
  if (MT == 0)
  {
    globalVariables->setExit(true);    
    cerr << "MeshX3DMerger: cannot merge!" << endl;  
    return;
  }

  if (MT->getTransform() != SFMatrix34f::IDENTITY)
  {
    // replacing the Group by a Transform.
    SFNode P = globalVariables->getParent();
    Transform *T = new Transform();
      
    // adding children.
    MFNode children = G->getChildren();
    for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
      G->addChild(*it);
      
    // setting parameters...
      
    // adding parents.
    MFAbstractNode parents = G->getParents();
    for (MFAbstractNode::iterator it = parents.begin(); it != parents.end(); ++it)
      (*it)->addChild(T);
     
    delete G;
  }
}

void MeshX3DMergerGroupingVisitor::leaveTransform(Transform *T) const
{
  globalVariables->popParent(); 

  MESH::Transform *MT = globalVariables->getNextTransform();
  if (MT == 0)
  {
    globalVariables->setExit(true);
    cerr << "MeshX3DMerger: cannot merge!" << endl; 
    return;
  }

  if (MT->getTransform() == SFMatrix34f::IDENTITY)
  {
    // replacing the Transform by a Group.
    SFNode P = globalVariables->getParent();
    Group *G = new Group();
          
    // adding children.
    MFNode children = T->getChildren();
    for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
      G->addChild(*it);
      
    // adding parents.
    MFAbstractNode parents = T->getParents();
    for (MFAbstractNode::iterator it = parents.begin(); it != parents.end(); ++it)
      (*it)->addChild(G);
    
    delete T;
  }
}

}
}
