#include "SceneSaverCoreVisitor.h"
#include "Scene.h"
#include "X3DNode.h"
#include "X3DTypesToSFString.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

SceneSaverCoreVisitor::SceneSaverCoreVisitor()
: CoreVisitor()
{
  // Enter functions.
  defineNewEnterFunction(&SceneSaverCoreVisitor::enterX3DNode);

  // Walk on functions.
  defineNewWalkOnFunction(&SceneSaverCoreVisitor::walkOnX3DNode);

  // Leave functions.
  defineNewLeaveFunction(&SceneSaverCoreVisitor::leaveX3DNode);

  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<SceneSaverGlobalVariables>();
}

SceneSaverCoreVisitor::~SceneSaverCoreVisitor()
{
}

void SceneSaverCoreVisitor::enterX3DNode(X3DNode *N) const
{
  SFString attr;
  SFString USE = globalVariables->getUSE(N);

  globalVariables->newLine();
  globalVariables->pushCurrentLine();  
  
  if (USE != "")
  {
    attr += USE;
    globalVariables->pushHasUSE(true);
    globalVariables->writeCurrentLine("<" + N->getTypeName() + attr);
  }
  else
  {
    attr += globalVariables->getDEF(N);
    globalVariables->pushHasUSE(false);    
    globalVariables->writeCurrentLine("<" + N->getTypeName() + attr + N->write());    
  } 

  globalVariables->addTab();
}

bool SceneSaverCoreVisitor::walkOnX3DNode(X3DNode *, SFAbstractNode) const
{
  //return true;
  return !(globalVariables->getHasUSE());  
}

void SceneSaverCoreVisitor::leaveX3DNode(X3DNode *N) const
{
  globalVariables->removeTab();
  
  if (globalVariables->hasChild())
    globalVariables->writeOpeningLine("/>");  
  else
  {
    globalVariables->writeOpeningLine(">");
    globalVariables->newLine();
    globalVariables->writeCurrentLine("</" + N->getTypeName() + ">");
  }  
  
  globalVariables->popLine();
  globalVariables->popHasUSE();  
}

}
}
