#include "TemplateMeshBuilderGroupingVisitor.h"
#include "X3DGroupingNode.h"
#include "StaticGroup.h"
#include "Transform.h"
#include "MESH_Transform.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

TemplateMeshBuilderGroupingVisitor::TemplateMeshBuilderGroupingVisitor()
: GroupingVisitor()
{
  // Enter functions.
  defineNewEnterFunction(&TemplateMeshBuilderGroupingVisitor::enterX3DGroupingNode);
  defineNewEnterFunction(&TemplateMeshBuilderGroupingVisitor::enterStaticGroup);
  defineNewEnterFunction(&TemplateMeshBuilderGroupingVisitor::enterTransform);

  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<TemplateMeshBuilderGlobalVariables>();
}

TemplateMeshBuilderGroupingVisitor::~TemplateMeshBuilderGroupingVisitor()
{
}

void TemplateMeshBuilderGroupingVisitor::enterX3DGroupingNode(X3DGroupingNode *N) const
{
  MESH::SFNode MN = globalVariables->getNode(N);
  if (MN == 0)
  {
    MN = new MESH::Transform();  
    globalVariables->addCoupleNode(N, MN);
  }
  globalVariables->pushNode(MN);
}

void TemplateMeshBuilderGroupingVisitor::enterStaticGroup(StaticGroup *S) const
{
  MESH::SFNode MS = globalVariables->getNode(S);
  if (MS == 0)
  {
    MS = new MESH::Transform();  
    globalVariables->addCoupleNode(S, MS);
  }
  globalVariables->pushNode(MS);
}

void TemplateMeshBuilderGroupingVisitor::enterTransform(Transform *T) const
{
  MESH::Transform *MT = dynamic_cast<MESH::Transform *>(globalVariables->getNode(T));
  if (MT == 0)
  {
    MT = new MESH::Transform();  
    globalVariables->addCoupleNode(T, MT);
  }
  globalVariables->pushNode(MT);

  SFMatrix34f transform(T->getTranslation(), T->getCenter(), T->getRotation(), T->getScaleOrientation(), T->getScale());
  MT->setTransform(transform);
}

}
}
