#include "Component.h"
#include "Type.h"
#include "SceneGraph.h"

#include <algorithm>
#include <iostream>

using namespace std;
using namespace X3DTK;

MFComponent Component::_componentList = MFComponent();

Component::Component(const SFString &name, const SFString &sceneGraph)
: _name(name), _refCount(0)
{
  //cout << "build Component " << name <<  ": " << this << endl;

  _sceneGraph = SceneGraph::getSceneGraph(sceneGraph);
  if (_sceneGraph == 0)
    _sceneGraph = new SceneGraph(sceneGraph);
  
  _sceneGraph->addOneReference();
  
  _componentList[getEncodedName()] = this;
}

Component::~Component()
{
  //cout << "destr Component " << _name << endl;
  
  SceneGraph::removeOneReference(_sceneGraph);
}

Component *Component::getComponent(const SFString &sceneGraph, const SFString &name)
{
  MFComponent::const_iterator it = _componentList.find(encodeKey(sceneGraph, name)); 
  if (it != _componentList.end())
    return (*it).second;
    
  //the type doesn't exist.
  return 0;  
}

void Component::addOneReference()
{
  ++_refCount;
  //cout << "add ref Component " << _name << " = " << _refCount << endl;
}

void Component::removeOneReference(Component *component)
{
  --component->_refCount;
  //cout << "remove ref Component " << component->getName() << " = " << component->_refCount << endl;
  
  if (component->_refCount == 0)
  {
    // delete component
    _componentList.erase(component->getEncodedName());
    delete component;
  }
}

SFString Component::encodeKey(const SFString &sceneGraph, const SFString &name)
{
  return sceneGraph + "_" + name;
}

void Component::addType(Type *type)
{
  _typeList[type->getName()] = type;
}
