#include "Type.h"
#include "X3DActor.h"

#include <algorithm>
#include <iostream>

using namespace X3DTK;
using namespace std;

int Type::count = 0;
MFType Type::_typeList = MFType();  
list<X3DActor *> Type::_actorList = list<X3DActor *>();
bool Type::_functionDefinition = false;

void Type::setParent(Type *parent)
{
  _parent = parent;
}

void Type::addOneReference()
{
  ++_refCount;
}

void Type::addOneReferenceRecursive()
{
  ++_refCount;
  if (_parent != 0)
    _parent->addOneReferenceRecursive();
}

void Type::removeOneReference(Type *type)
{
  --type->_refCount;
  
  //removes the instance of the parents.
  if (type->_parent != 0)
    removeOneReference(type->_parent);
  
  if (type->_refCount == 0)  
  {
    //remove in the list.      
    _typeList.erase(type->getEncodedName());
    delete type;
  }
}

Type *Type::getTypeOfName(const SFString &sceneGraph, const SFString &component, const SFString &name)
{
  MFType::const_iterator it = _typeList.find(encodeKey(sceneGraph, component, name)); 
  if (it != _typeList.end())
    return (*it).second;
    
  //the type doesn't exist.
  return 0;  
}

Type *Type::getTypeOfId(int id)
{
  // iterate the Types
  for (MFType::const_iterator it = _typeList.begin(); it != _typeList.end(); ++it)
    if ((*it).second->getId() == id)
      return (*it).second;
      
  return 0;      
}
void Type::beginFunctionDefinition()
{
  _functionDefinition = true;
}

void Type::endFunctionDefinition()
{
  _functionDefinition = false;
}

SFString Type::encodeKey(const SFString &sceneGraph, const SFString &component, const SFString &type)
{
  return sceneGraph + "_" + component + "_" + type;
}

void Type::recomputeIds()
{
}

void Type::printDerivationTree()
{
  cout << "Derivation tree:" << endl;
  SFString tab("");
  //finding the nodes which have no parent.
  for (MFType::const_iterator it = _typeList.begin(); it != _typeList.end(); ++it)
  {
    if ((*it).second->_parent == 0)
    {
      (*it).second->printDerivationTree(tab);
    } 
  }    
  cout << "Derivation tree finished" << endl;
}

Type::Type(const SFString &name, const SFString &component, const SFString &sceneGraph)
: _name(name), _refCount(0)
{
  //cout << "build " << name <<  ": " << this << endl;

  //component update.
  _component = Component::getComponent(sceneGraph, component);
  if (_component == 0)
    _component = new Component(component, sceneGraph);

  //component referenced by a new Type.
  _component->addOneReference();
}

Type::~Type()
{
  //cout << "destr Type " << _name << endl;
  Component::removeOneReference(_component);
  _childList.clear();
}

void Type::defineTypeName(Type **type, const SFString &name, const SFString &component, const SFString &sceneGraph)
{
  // Getting the name of the component.
  SFString tcomponent = component;
  if (tcomponent == "")
    tcomponent = (*type)->_component->getName();
  
  SFString tsceneGraph = sceneGraph;
  if ((tsceneGraph == "") && (*type != 0))
    tsceneGraph = (*type)->_component->_sceneGraph->getName();
     
  Type *t = getTypeOfName(tsceneGraph, tcomponent, name);
  
  if (t == 0)
  {
    t = new Type(name, tcomponent, tsceneGraph);
    t->_id = count;
    t->_parent = *type;  

    //adding new name.
    SFString nodeName = encodeKey(tsceneGraph, tcomponent, name);
       
    if (*type != 0)    
      (*type)->_childList[nodeName] = t;
      
      _typeList[nodeName] = t;

    if (!_functionDefinition)
    {
      ++count;
      //advertising actors
      for (list<X3DActor *>::iterator itActor = _actorList.begin(); itActor != _actorList.end(); ++itActor) 
        (*itActor)->addType(t);
    }
  }
 
  //the type t is referenced by a new Node.
  t->addOneReference();
  *type = t;
}

void Type::addActor(X3DActor *actor)
{
  //updating proxy list
  _actorList.push_back(actor);
  
  //adding all types
  for (MFType::const_iterator it = _typeList.begin(); it != _typeList.end(); ++it)
    actor->addType((*it).second);
}

void Type::removeActor(X3DActor *actor)
{
  _actorList.remove(actor);
}

void Type::printDerivationTree(SFString tab) const
{
  cout << tab << _name << ": " << _refCount << ", " << _component->getName() << ", " << _component->getSceneGraph()->getName() << ": " << _component->_refCount << endl;
  tab += "  ";  

  for (MFType::const_iterator it = _childList.begin(); it != _childList.end(); ++it)
    (*it).second->printDerivationTree(tab);
}
