#include "JPGImageType.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef JPEG

extern "C" {
#include <setjmp.h>
#include <jpeglib.h>
}

#endif

#include <iostream>

using namespace std;

namespace X3DTK {

JPGImageType::JPGImageType()
{
}

bool JPGImageType::load(const char *fileName)
{
  bool retCode = false;

#ifdef JPEG

  struct local_error_mgr
  {
    struct jpeg_error_mgr pub;
    jmp_buf setjmp_buffer;
  };

  unsigned char *destData;
  typedef struct local_error_mgr *local_error_ptr;
  struct local_error_mgr jerr;
  struct jpeg_decompress_struct cinfo;
  FILE *infile;
  JSAMPARRAY buffer;

  int row_stride;

  if((infile = fopen(fileName, "rb")))
  {
    cinfo.err = jpeg_std_error(&jerr.pub);
    if(setjmp(jerr.setjmp_buffer))
    {
      jpeg_destroy_decompress(&cinfo);
      fclose(infile);
      return 0;
    }

    jpeg_create_decompress(&cinfo);
    jpeg_stdio_src(&cinfo, infile);
    jpeg_read_header(&cinfo, TRUE);
    jpeg_start_decompress(&cinfo);

    if(allocate(cinfo.output_components, cinfo.output_width, cinfo.output_height))
    {
      destData = data + cinfo.output_width*cinfo.output_height*cinfo.output_components;
      row_stride = cinfo.output_width*cinfo.output_components;
      buffer = (*cinfo.mem->alloc_sarray) ((j_common_ptr) & cinfo, JPOOL_IMAGE, row_stride, 1);
            
      while(cinfo.output_scanline < cinfo.output_height)
      {
        destData -= row_stride;
        jpeg_read_scanlines(&cinfo, buffer, 1);
        memcpy(destData, *buffer, row_stride);
      }
 
      retCode = true;
    }
    else
      retCode = false;

    jpeg_finish_decompress(&cinfo);
    jpeg_destroy_decompress(&cinfo);
    fclose(infile);
  }

#else
  cerr << "sorry, jpeg not supported by your platform!" << endl;
#endif

 
  return retCode;
}

}
